from zyklus import Zyklus
import unittest

import time


class HelperObj(object):
    def __init__(self, initial):
        self.curr = initial

    def add(self, what):
        self.curr += what

    def multiply(self, what):
        self.curr = self.curr * what

    def result(self):
        return self.curr
import threading
class ZyklusTest(unittest.TestCase):
    def setUp(self):
        self.zyklus = Zyklus()
        self.zyklusThread = threading.Thread(target=self.zyklus.loop)
        self.zyklusThread.start()

        self.helperObj = HelperObj(2)

    def tearDown(self):
        self.zyklus.terminate()

    def test_loop(self):
        # import threading
        # t = threading.Thread(target=self.zyklus.loop)
        # t.start()
        self.zyklus.post(lambda: self.helperObj.add(1))
        time.sleep(0.1)
        self.assertEqual(3, self.helperObj.result())

        self.zyklus.post(lambda: self.helperObj.add(1))
        self.zyklus.post(lambda: self.helperObj.add(1))
        time.sleep(0.1)
        self.assertEqual(5, self.helperObj.result())

    def test_terminate(self):
        self.zyklus.terminate()
        initial = self.helperObj.result()
        self.zyklus.post(lambda: self.helperObj.add(1))
        time.sleep(0.1)
        self.assertEqual(initial, self.helperObj.result())
        self.assertFalse(self.zyklusThread.isAlive())
        self.assertTrue(self.zyklus.terminated)

    def test_post(self):
        target = self.helperObj
        self.zyklus.post(lambda: target.add(1))
        self.zyklus.post(lambda: target.multiply(2))
        time.sleep(0.1)
        self.assertEqual(target.result(), 6)

        try:
            self.zyklus.post(self.helperObj)
            raise AssertionError()
        except AssertionError:
            pass
            ##ok

    def test_is_callable(self):
        self.assertTrue(Zyklus.is_callable(lambda a: 1))
        self.assertFalse(Zyklus.is_callable(self.helperObj))
        self.helperObj.__call__ = lambda a,b: 1
        self.assertTrue(Zyklus.is_callable(self.helperObj))

    def test_ensure_postable(self):
        try:
            self.zyklus.ensure_postable(self.helperObj)
            raise AssertionError()
        except AssertionError:
            pass
            ##ok
        self.assertTrue(self.zyklus.ensure_postable(lambda : 1))

    def test_post_delayed(self):
        initialValue = self.helperObj.result()
        expectedResult = (initialValue + 5) * 10
        self.zyklus.post_delayed(lambda: self.helperObj.multiply(10), 0.2)
        self.zyklus.post_delayed(lambda: self.helperObj.add(5), 0.1)
        self.assertEqual(initialValue, self.helperObj.result())
        time.sleep(0.4)
        self.assertEqual(expectedResult, self.helperObj.result())

    def test_post_delayed_cancellation(self):
        initialValue = self.helperObj.result()
        expectedResult = (initialValue + 5)
        multPost = self.zyklus.post_delayed(lambda: self.helperObj.multiply(10), 0.2)
        self.zyklus.post_delayed(lambda: self.helperObj.add(5), 0.1)
        self.assertEqual(initialValue, self.helperObj.result())
        multPost.cancel()
        time.sleep(0.4)
        self.assertEqual(expectedResult, self.helperObj.result())

    def test_exec_callable(self):
        self.zyklus.exec_callable(lambda: self.helperObj.add(9))
        self.assertEqual(11, self.helperObj.result())


import logging
import threading
try:
    import Queue
except ImportError:
    import queue as Queue

logger = logging.getLogger(__name__)


class DelayedPost(object):
    def __init__(self, timer):
        self.__timer = timer

    def cancel(self):
        self.__timer.cancel()


class Zyklus(object):
    def __init__(self):
        self.eventQueue = Queue.Queue()
        self.terminated = False

    def loop(self):
        if self.terminated:
            raise ValueError("Already terminated")

        logger.debug("Starting event loop")
        while not self.terminated:
            self.exec_callable(self.eventQueue.get())

        logger.debug("Event loop terminated")

    def terminate(self):
        logger.debug("Going to terminate loop")
        if not self.terminated:
            self.post(self.__do_terminate)

    def __do_terminate(self):
        logger.debug("Terminating loops")
        self.terminated = True

    def post(self, clbl):
        if self.__class__.ensure_postable(clbl):
            if not self.terminated:
                self.eventQueue.put(clbl)
            else:
                logger.warning("Loop already terminated, won't post")
        else:
            pass

    @staticmethod
    def is_callable(clbl):
        return hasattr(clbl, '__call__') or callable(clbl)

    @classmethod
    def ensure_postable(cls, clbl):
        assert cls.is_callable(clbl), "Can't post non-callables"
        return True

    def post_delayed(self, clbl, t):
        assert type(t) in (int, float)
        self.__class__.ensure_postable(clbl)
        t = threading.Timer(t, self.post, args=(clbl,))
        t.start()
        return DelayedPost(t)

    def exec_callable(self, clbl):
        clbl()


#!/usr/bin/env python
from __future__ import print_function
from setuptools import setup, find_packages
import zyklus

setup(
    name='zyklus',
    version=zyklus.__version__,
    url='http://github.com/tgalal/zyklus/',
    license='MIT',
    author='Tarek Galal',
    tests_require=[],
    install_requires = [],
    author_email='tare2.galal@gmail.com',
    description='A simple event loop library',
    #long_description=long_description,
    packages= find_packages(),
    include_package_data=True,
    platforms='any',
    # test_suite='',
    classifiers = [
        'Programming Language :: Python',
        'Development Status :: 4 - Beta',
        'Natural Language :: English',
        #'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules'
        ],
)
